#ifndef __FRAMEBUFFER
#define __FRAMEBUFFER

//#include "glheaders.h"
#include "texture.h"

namespace gl
{
	class framebuffer
	{
		struct info_t 
		{
			GLenum target;
			GLuint name;
			info_t(GLenum target) : target(target) { glGenFramebuffers(1,&name);  }
			~info_t() { glDeleteFramebuffers(1,&name); }
		};
		typedef std::shared_ptr<struct info_t> shared_info;
		shared_info info;
	public:
		framebuffer()				: info(new info_t(GL_FRAMEBUFFER)) { ; }
		framebuffer(GLenum target)	: info(new info_t(target)) { ; }

		GLuint name() const { return info->name; }

		      framebuffer& bind(GLenum target = GL_FRAMEBUFFER)  { glBindFramebuffer(info->target=target, info->name); return *this; }
		const framebuffer& bind() const { glBindFramebuffer(info->target, info->name); return *this; }

		framebuffer& attach(GLenum slot, const texture& tex) 
		{ 
			bind(); 
			//dprintf("bind %04x to fb %p : gl error: %04x\n", info->target, this, glGetError());
			glFramebufferTexture2D(info->target, slot, tex.target(), tex.name(), 0); 
			//dprintf("gl error: %04x\n", glGetError());
			return *this; 
		}
		class attachment
		{
			framebuffer* parent;
			GLenum slot;
			attachment(framebuffer* parent, GLenum slot) : parent(parent), slot(slot) { ; }
		public:
			framebuffer& attach(texture& tex) { return parent->attach(slot, tex); }
			friend class framebuffer;
		};
		attachment color(int i) { return attachment(this,GL_COLOR_ATTACHMENT0+i); }
		attachment depth() { return attachment(this,GL_DEPTH_ATTACHMENT); }
		
		GLenum status() const { bind(); return glCheckFramebufferStatus(info->target); }

		static void bind_default() { glBindFramebuffer(GL_FRAMEBUFFER, 0); }
	};
}

#endif
