#ifdef _WIN32
#include <Windows.h>
#endif
#include <GL/glew.h>
#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "fmodex_vc.lib")

#include <SDL/SDL.h>
#include "stb_image.c"
#include <string>
#include <iostream>
#include <fmod.hpp>

#include "texture.h"
#include "framebuffer.h"

SDL_Event event;
SDL_Surface *screen = NULL;

int window_w = 1280;
int window_h = 720;

//void drawImage(SDL_Surface* image) { 
//}
void drawTexture(gl::texture tex, bool flip = true) { 

	glMatrixMode(GL_PROJECTION_MATRIX);
	glLoadIdentity();
	//if (flip)
	//	glOrtho(-1,1,1,-1,-1,1);
	//else
	//	glOrtho(-1,1,-1,1,-1,1);
	glMatrixMode(GL_MODELVIEW_MATRIX);
	glLoadIdentity();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	gl::texture::bind tex_bind(tex);
	glColor3d(1,1,1);
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2d(0, flip); glVertex2d(-1,-1);
	glTexCoord2d(0,!flip); glVertex2d(-1,+1);
	glTexCoord2d(1, flip); glVertex2d(+1,-1);
	glTexCoord2d(1,!flip); glVertex2d(+1,+1);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

}

void drawSprite(gl::texture tex, float x, float y, float z, bool flip = true) { 

	glMatrixMode(GL_PROJECTION_MATRIX);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW_MATRIX);
	glLoadIdentity();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	gl::texture::bind tex_bind(tex);
	glColor3d(1,1,1);
	glTranslatef(x,y,z);
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2d(0, flip); glVertex2d(-1,-1);
	glTexCoord2d(0,!flip); glVertex2d(-1,+1);
	glTexCoord2d(1, flip); glVertex2d(+1,-1);
	glTexCoord2d(1,!flip); glVertex2d(+1,+1);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

}
SDL_Surface *loadImage(const std::string &filename) {
	std::cout << "loading " << filename << std::endl;

	int x, y, comp;
	unsigned char *data;
	uint32_t rmask, gmask, bmask, amask;
	SDL_Surface *rv;

	FILE *file = fopen(filename.c_str(), "rb");
	if(!file) {
		std::cerr << "Error: missing image: " << filename << std::endl;
		exit;
	}

	data = stbi_load_from_file(file, &x, &y, &comp, 0);
	fclose(file);

	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	#endif

	if (comp == 4) {
        	rv = SDL_CreateRGBSurface(0, x, y, 32, rmask, gmask, bmask, amask);
	} else if (comp == 3) {
        	rv = SDL_CreateRGBSurface(0, x, y, 24, rmask, gmask, bmask, 0);
	} else {
        	stbi_image_free(data);
	        return 0;
    	}

	memcpy(rv->pixels, data, comp * x * y);
	stbi_image_free(data);

	return rv;
}
gl::texture loadTexture(const std::string &filename) {
	std::cout << "loading " << filename << std::endl;

	int x, y, comp;
	unsigned char *data;
	uint32_t rmask, gmask, bmask, amask;
	gl::texture tex;

	FILE *file = fopen(filename.c_str(), "rb");
	if(!file) {
		std::cerr << "Error: missing image: " << filename << std::endl;
		exit;
	}

	data = stbi_load_from_file(file, &x, &y, &comp, 0);
	fclose(file);

	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	#endif

	tex.size(x,y);
	if (comp == 4) {
		tex.format(GL_RGBA8).load_image(GL_RGBA,GL_UNSIGNED_BYTE,data);
        	//rv = SDL_CreateRGBSurface(0, x, y, 32, rmask, gmask, bmask, amask);
	} else if (comp == 3) {
		tex.format(GL_RGB8).load_image(GL_RGB,GL_UNSIGNED_BYTE,data);
        	//rv = SDL_CreateRGBSurface(0, x, y, 24, rmask, gmask, bmask, 0);
	}
	else
		tex.load_blank(GL_RGB);
	stbi_image_free(data);

	return tex;
}

void init_SDL_GL()
{
	SDL_Init( SDL_INIT_VIDEO );
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	screen = SDL_SetVideoMode(window_w, window_h, 32, SDL_OPENGL | SDL_FULLSCREEN); 
	glewInit();
	std::cout << "screen: " << screen << "\n";
	SDL_WM_SetCaption("fastintro", 0);
	SDL_ShowCursor(0);
}

int main(int argc, char* argv[]) {
	init_SDL_GL();
	// audio
	FMOD::System *system;
	FMOD::Sound *music;
	FMOD::Channel *channel = 0;

	FMOD::System_Create(&system);
	system->setOutput(FMOD_OUTPUTTYPE_ALSA);
	system->init(1, FMOD_INIT_NORMAL, 0);
	system->createSound("cos-other.ogg", FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM, 0, &music);

	// framebuffer setup
	auto hi_res_tex = gl::texture().size(1280,720).format(GL_RGBA8).load_blank(GL_RGB);
	auto lo_res_tex = gl::texture().size(320,200).format(GL_RGBA8).load_blank(GL_RGB);
	auto hi_res = gl::framebuffer().color(0).attach(hi_res_tex);
	auto lo_res = gl::framebuffer().color(0).attach(lo_res_tex);

	gl::texture* fb_tex = &hi_res_tex;
	gl::framebuffer* fb = &hi_res;
	{
		gl::texture::bind bind(lo_res_tex);
		glTexParameteri(lo_res_tex.target(), GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(lo_res_tex.target(), GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	}


	// loading
	gl::texture loading = loadTexture("images/loading.png");
	gl::texture space = loadTexture("images/1-space-hd.png");
	gl::texture scumbar = loadTexture("images/2-scumbar-hd.png");
	gl::texture grog = loadTexture("images/3-grog-hd.png");
	gl::texture eightbitglasses = loadTexture("images/4-8-bit-glasses-hd.png");
	gl::texture funkytown = loadTexture("images/5-funky-town.png");
	gl::texture nextday = loadTexture("images/next-day.png");
	gl::texture seacow = loadTexture("images/manateeb.png");
	gl::texture swimingdolphin = loadTexture("images/9-swiming-dolphin-hd.png");
	gl::texture dolphin = loadTexture("images/delfin-sprite.png");
	gl::texture dolphinflipped = loadTexture("images/delfin-sprite-flipped.png");
	gl::texture glasses = loadTexture("images/briller-sprite.png");
	gl::texture pitcher = loadTexture("images/pitcher.png");
	gl::texture star1 = loadTexture("images/stjerne_v.png");
	gl::texture star2 = loadTexture("images/stjerne_h.png");
	gl::texture longday = loadTexture("images/long-day.png");
	gl::texture bitgrog = loadTexture("images/one-8bit-grog.png");
	gl::texture walk = loadTexture("images/walking-to-scumm-bar.png");

	glClearColor(0,0,0,1);

	system->playSound(FMOD_CHANNEL_FREE, music, false, &channel);
	int then = SDL_GetTicks();
	do {
		SDL_PollEvent(&event);
		int t=SDL_GetTicks()-then; // timer ms

		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0xFF, 0xFF, 0xFF));
		fb->bind();
		// scenes
		glViewport(0,0, fb_tex->width(),fb_tex->height());
		float localtime = 0;
		if( t <= 4000)
			drawTexture(loading);
		else if(t <= 8000)
			drawTexture(longday);
		else if(t <= 12000)
		{
			drawTexture(space);
			localtime=(t/2000.0);
			std::cout << cos(localtime) << std::endl;
		    drawSprite(dolphin,localtime*0.4-1.5,0.4*sin(localtime),0);
		}
		else if(t <= 16000)
			drawTexture(walk);
		else if(t <= 20000){
			drawTexture(scumbar);
			drawSprite(dolphin,0.2,-0.1,0);
			drawSprite(pitcher,0.5,0,0);
		}
		else if(t <= 24000)
			drawTexture(bitgrog);
		else if(t <= 28000)
		{
			drawTexture(grog);
			drawSprite(dolphin,0.2,-0.1,0.5);
		}
		else if(t <= 32000)
		{
			fb = &lo_res;
			fb_tex = &lo_res_tex;
			glViewport(0,0, fb_tex->width(),fb_tex->height());
			drawTexture(eightbitglasses);
			drawSprite(glasses,0,0,0);
			drawSprite(star1,0,0,0);
			drawSprite(star2,0,0,0);

		}
		else if(t <= 36000)
		{
			drawTexture(funkytown);
			drawSprite(dolphin,0,-0.2,0);
		}	
		else if(t <= 40000)
		{
			glClear(GL_COLOR_BUFFER_BIT);
			drawTexture(nextday);
		}
		else if(t <= 44000)
		{
			fb = &hi_res;
			fb_tex = &hi_res_tex;
			glViewport(0,0, fb_tex->width(),fb_tex->height());
			drawTexture(seacow);
			drawSprite(dolphinflipped,0,0,0);
		}
		else if(t <= 48000)
			drawTexture(eightbitglasses);
		else if(t <= 52000)
			drawTexture(swimingdolphin);
		else
			break;

		fb->bind_default();
		glViewport(0,0, window_w,window_h);
		drawTexture(*fb_tex,false);

		system->update(); // update music
		SDL_GL_SwapBuffers();
		SDL_Delay(10);

	} while(event.type!=SDL_KEYDOWN);

	// clean
	// fmod
	music->release();
	system->close();
	system->release();

	SDL_Quit();
	return 0;
}
