#include <SDL/SDL.h>
#include "stb_image.c"
#include <string>
#include <iostream>
#include "fmodapi44004linux/api/inc/fmod.hpp"

SDL_Event event;
SDL_Surface *screen = NULL;

void drawImage(SDL_Surface* image) { 
	SDL_BlitSurface(image, 0, screen, 0);
}

SDL_Surface *loadImage(const std::string &filename) {
	std::cout << "loading " << filename << std::endl;

	int x, y, comp;
	unsigned char *data;
	uint32_t rmask, gmask, bmask, amask;
	SDL_Surface *rv;

	FILE *file = fopen(filename.c_str(), "rb");
	if(!file) {
		std::cerr << "Error: missing image: " << filename << std::endl;
		exit;
	}

	data = stbi_load_from_file(file, &x, &y, &comp, 0);
	fclose(file);

	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	#endif

	if (comp == 4) {
        	rv = SDL_CreateRGBSurface(0, x, y, 32, rmask, gmask, bmask, amask);
	} else if (comp == 3) {
        	rv = SDL_CreateRGBSurface(0, x, y, 24, rmask, gmask, bmask, 0);
	} else {
        	stbi_image_free(data);
	        return 0;
    	}

	memcpy(rv->pixels, data, comp * x * y);
	stbi_image_free(data);

	return rv;
}

int main() {
	// audio
	FMOD::System *system;
	FMOD::Sound *music;
	FMOD::Channel *channel = 0;

	FMOD::System_Create(&system);
	system->setOutput(FMOD_OUTPUTTYPE_ALSA);
	system->init(1, FMOD_INIT_NORMAL, 0);
	system->createSound("cos-other.ogg", FMOD_SOFTWARE | FMOD_2D | FMOD_CREATESTREAM, 0, &music);
	system->playSound(FMOD_CHANNEL_FREE, music, false, &channel);

	// window
	screen = SDL_SetVideoMode(640, 480, 0, 0); /* | SDL_FULLSCREEN); */
	SDL_WM_SetCaption("fastintro", 0);
	SDL_ShowCursor(0);

	// loading
	SDL_Surface *test = loadImage("images/test.png");

	do {
		SDL_PollEvent(&event);
		int t=SDL_GetTicks(); // timer ms

		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0xFF, 0xFF, 0xFF));

		// scenes
		if(t >= 0 && t <= 2000)
			//SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0xFF, 0x00, 0x00));
			drawImage(test);
		else if(t <= 4000)
			SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0x00, 0xFF, 0x00));
		else if(t <= 6000)
			SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0x00, 0x00, 0xFF));
		else if(t >= 6000)
			break;

		system->update(); // update music
		SDL_Flip(screen);
	} while(event.type!=SDL_KEYDOWN);

	// clean
	SDL_FreeSurface(test);
//	AudioStreamDeinit();

	// fmod
	music->release();
	system->close();
	system->release();

	// quit
	SDL_Quit();
	return 0;
}
