#include <SDL/SDL.h>
#include "stb_image.c"
#include <string>
#include <iostream>

SDL_Event event;
SDL_Surface *screen = NULL;

void drawImage(SDL_Surface* image) { 
	SDL_BlitSurface(image, 0, screen, 0);
}

SDL_Surface *loadImage(const std::string &filename) {
	std::cout << "loading " << filename << std::endl;

	int x, y, comp;
	unsigned char *data;
	uint32_t rmask, gmask, bmask, amask;
	SDL_Surface *rv;

	FILE *file = fopen(filename.c_str(), "rb");
	if(!file) {
		std::cerr << "Error: missing image: " << filename << std::endl;
		exit;
	}

	data = stbi_load_from_file(file, &x, &y, &comp, 0);
	fclose(file);

	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0xff000000;
		gmask = 0x00ff0000;
		bmask = 0x0000ff00;
		amask = 0x000000ff;
	#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		amask = 0xff000000;
	#endif

	if (comp == 4) {
        	rv = SDL_CreateRGBSurface(0, x, y, 32, rmask, gmask, bmask, amask);
	} else if (comp == 3) {
        	rv = SDL_CreateRGBSurface(0, x, y, 24, rmask, gmask, bmask, 0);
	} else {
        	stbi_image_free(data);
	        return 0;
    	}

	memcpy(rv->pixels, data, comp * x * y);
	stbi_image_free(data);

	return rv;
}

int main() {
	screen = SDL_SetVideoMode(640, 480, 0, 0); /* | SDL_FULLSCREEN); */
	SDL_WM_SetCaption("fastintro", 0);
	SDL_ShowCursor(0);

	// loading
	SDL_Surface *loading = loadImage("images/0-loading.png");
	SDL_Surface *space = loadImage("images/1-space.png");
	SDL_Surface *scumbar = loadImage("images/2-scumbar.png");
	SDL_Surface *grog = loadImage("images/3-grog.png");
	SDL_Surface *eightbitglasses = loadImage("images/4-8-bit-glasses.png");
	SDL_Surface *funkytown = loadImage("images/5-funky-town.png");
	SDL_Surface *nextday = loadImage("images/6-next-day.png");
	SDL_Surface *seacow = loadImage("images/7-sea-cow.png");
	SDL_Surface *notagain = loadImage("images/8-not-again.png");
	SDL_Surface *swimingdolphin = loadImage("images/9-swiming-dolphin.png");

	do {
		SDL_PollEvent(&event);
		int t=SDL_GetTicks(); // timer ms

		SDL_FillRect(screen, &screen->clip_rect, SDL_MapRGB(screen->format, 0xFF, 0xFF, 0xFF));

		// scenes
		if(t >= 0 && t <= 2000)
			drawImage(loading);
		else if(t <= 4000)
			drawImage(space);
		else if(t <= 6000)
			drawImage(scumbar);
		else if(t <= 8000)
			drawImage(grog);
		else if(t <= 10000)
			drawImage(eightbitglasses);
		else if(t <= 12000)
			drawImage(funkytown);
		else if(t <= 14000)
			drawImage(nextday);
		else if(t <= 16000)
			drawImage(seacow);
		else if(t <= 18000)
			drawImage(notagain);
		else if(t <= 20000)
			drawImage(swimingdolphin);
		else
			break;

		SDL_Flip(screen);
	} while(event.type!=SDL_KEYDOWN);

	// clean
	SDL_FreeSurface(loading);
    SDL_FreeSurface(space);
    SDL_FreeSurface(scumbar);
    SDL_FreeSurface(grog);
    SDL_FreeSurface(eightbitglasses);
    SDL_FreeSurface(funkytown);
    SDL_FreeSurface(nextday);
    SDL_FreeSurface(seacow);
    SDL_FreeSurface(notagain);
    SDL_FreeSurface(swimingdolphin);


	SDL_Quit();
	return 0;
}
