#ifndef __SHADER_H
#define __SHADER_H

#ifdef _WIN32
#include <Windows.h>
#endif
#include <GL/glew.h>
//#include "glheaders.h"
#include "../demo/debugio.h"
#include <utility>

namespace glsl
{
	class shader
	{
		GLuint id;
		shader(const shader& sh);
		shader& operator=(const shader& sh);
	public:
		shader() : id(0) { ; }
		shader(const char* source, GLenum type);
		shader(shader&& sh) : id(sh.id) 
		{ 
			sh.id = 0; 
			//dprintf("shader %d moved\n", id); 
		}
		~shader() 
		{ 
			glDeleteShader(id); 
			//dprintf("shader %d deleted\n", id); 
		}

		//shader& operator=(shader&& sh) { glDeleteShader(id); id = sh.id; sh.id = 0; }

		static shader from_file(const char* filename, GLenum type);
		friend class program;
	};

	class program
	{
		GLuint id;
		program(const program& p);
		program& operator=(const program& p);
	public:
		program() : id(glCreateProgram()) { ; }
		program(program&& p) : id(p.id) { p.id = 0; }
		~program() { glDeleteProgram(id); }

		program& operator=(program&& p) { id = p.id; p.id = 0; }

		program attach(const shader& sh) 
		{
			glAttachShader(id, sh.id); 
			//dprintf("shader %d attached\n", sh.id); 
			return std::move(*this); 
		}

		program link();

		GLuint uniform(const char* name) const { return glGetUniformLocation(id, name); }

		static void use_default() { glUseProgram(0); }
		void use() const { glUseProgram(id); }
	};
}

#endif