#ifndef __TEXTURE
#define __TEXTURE

//#include "glheaders.h"
#include <memory>

namespace gl
{
	class texture
	{
		struct info_t
		{
			GLenum target;
			GLuint name;
			GLenum iformat;
			GLuint width;
			GLuint height;
			GLuint depth;
			info_t(GLenum target) : target(target), iformat(GL_RGB8), width(0), height(1), depth(1) { glGenTextures(1,&name); }
			~info_t() { if (name != 0) glDeleteTextures(1,&name); }
		};
		typedef std::shared_ptr<struct info_t> shared_info;
		shared_info info;
	public:
		class bind
		{
			GLenum target;
			GLint prev_bind;
			GLuint tex_slot;
			bind() { ; }
			static GLenum _binding(GLenum target)
			{
				switch (target)
				{
				case GL_TEXTURE_2D:			return GL_TEXTURE_BINDING_2D;
				case GL_TEXTURE_RECTANGLE:	return GL_TEXTURE_BINDING_RECTANGLE;
				case GL_TEXTURE_CUBE_MAP:	return GL_TEXTURE_BINDING_CUBE_MAP;
				case GL_TEXTURE_1D:			return GL_TEXTURE_BINDING_1D;
				case GL_TEXTURE_3D:			return GL_TEXTURE_BINDING_3D;
				default:
					printf("uncommon texture target used. please update code and recompile!\n");
					return GL_NONE;
				}
			}
		public:
			bind(const texture& tex, GLuint texture_slot = GL_TEXTURE0) 
				: target(tex.target()), tex_slot(texture_slot)
			{
				//glActiveTexture(tex_slot);
				glGetIntegerv(_binding(target),&prev_bind);
				//printf("bind texture %d to %d, prev %d\n", tex_slot-GL_TEXTURE0, tex.name(), prev_bind);
				glBindTexture(target, tex.name());
				//glActiveTexture(GL_TEXTURE0);
			}
			~bind() 
			{ 
				//printf("bind texture %d back to %d\n", tex_slot-GL_TEXTURE0, prev_bind);
				//glActiveTexture(tex_slot);
				glBindTexture(target, prev_bind); 
				//glActiveTexture(GL_TEXTURE0);
			}
		};
		texture()				: info(new info_t(GL_TEXTURE_2D))	{ ; }
		texture(GLenum target)	: info(new info_t(target))			{ ; }
#ifdef _SDL_H
		texture(SDL_Surface* image) : info(new info_t(GL_TEXTURE_2D))
		{
			size(image->w,image->h);
			format(GL_RGBA8);
			load_image(GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);
			std::cout << image->w << "x" << image->h << ": " << int(image->format->BitsPerPixel) << "bpp " << std::hex << image->format->Rmask << std::dec << "\n";
		}
#endif
		//texture(const texture& a) : info(a.info) { ; }

		GLenum target() const { return info->target; }
		GLuint name()   const { return info->name; }

		//      texture& bind()       { glBindTexture(info->target, info->name); return *this; }
		//const texture& bind() const { glBindTexture(info->target, info->name); return *this; }

		texture& format(GLenum iformat) { info->iformat = iformat; return *this; }
		texture& size(GLuint width, GLuint height)	{ info->width = width; info->height = height; return *this; }
		texture& size(const GLuint sizev[])			{ return size(sizev[0], sizev[1]); }
		texture&  width(GLuint w) { info->width  = w; return *this;}
		texture& height(GLuint h) { info->height = h; return *this;}
		texture&  depth(GLuint d) { info->depth  = d; return *this; }

		GLuint width()  const { return info->width; }
		GLuint height() const { return info->height; }
		
		texture& load_image(GLenum dataformat, GLenum datatype, const void* data) 
		{ 
			bind bind0(*this,GL_TEXTURE0); 
			glTexImage2D(info->target,0, info->iformat,info->width,info->height, 0, dataformat,datatype,data); 
			std::cout << std::hex << glGetError() << std::dec <<  "\n";
			glTexParameteri(info->target, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(info->target, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			return *this; 
		}
		texture& load_image1D(GLenum dataformat, GLenum datatype, const void* data)
		{
			info->target = GL_TEXTURE_1D;
			bind bind0(*this,GL_TEXTURE0);
			glTexImage1D(info->target, 0, info->iformat, info->width, 0, dataformat,datatype,data);
			return *this;
		}
		texture& load_image3D(GLenum dataformat, GLenum datatype, const void* data)
		{
			info->target = GL_TEXTURE_3D;
			bind bind0(*this,GL_TEXTURE0);
			glTexImage3D(info->target, 0, info->iformat, info->width,info->height,info->depth, 0, dataformat,datatype,data);
			return *this;
		}
		texture& load_blank(GLenum format) { return load_image(format,GL_BYTE,nullptr); }
		/*{ 
			bind(); 
			//int boundtex = -1;
			//glGetIntegerv(GL_TEXTURE_BINDING_RECTANGLE, &boundtex);
			//printf("creating %dx%d texture at target %04x, name %d with format %04x\n", 
			//	info->width, info->height, info->target, boundtex, info->iformat);
			//printf("error before: %04x\n", glGetError());
			glTexImage2D(info->target,0, info->iformat,info->width,info->height, 0, format,GL_BYTE,nullptr);  
			//printf("error after:  %04x\n", glGetError());
			return std::move(*this); 
		}*/


		static void bind_default() { glBindTexture(GL_TEXTURE_2D, 0); }
	};
}

#endif
